/*jslint nomen: true, indent: 2, maxerr: 3, maxlen: 80*/
/*global window, RSVP, rJS, jIO*/
(function (window, RSVP, rJS, jIO) {
 "use strict";
 rJS(window)
 
 .setState({
   docs : null,
   indexes : null
 })
 
 .ready(function(){
   var docs_db = jIO.createJIO({
                type : "indexeddb",
                database : "mynij.all_docs"
              });
    var indexes_db = jIO.createJIO({
                type : "indexeddb",
                database : "mynij.all_indexes"
              });
    return this.changeState({
      docs : docs_db,
      indexes : indexes_db
    });
 })
 
 .declareMethod("get_all_available_indexes", function(){
   //TODO later
  })
  
  .declareMethod("get_doc", function(doc_name){ //tested OK returns {links:[]}
    return this.state.docs.get(doc_name);
  })
  
  .declareMethod("get_index", function(index_name){ //tested OK
    return this.state.indexes.get(index_name)
      .push(function(result_index){
        var raw_index,
            index,
            key,
            doc,
            doc_list;
            
        doc_list = result_index.docs;
        index = elasticlunr(function () {
          this.use(elasticlunr.fr);
          this.addField('title');
          this.addField('body');
          this.addField('link');
          this.setRef('id'); //id = index.x."/a/b" where index = index name, x = doc name, "/a/b" = item id in x  
        });
        
        raw_index = JSON.parse(result_index.index);
        for (key in raw_index.documentStore.docs){
          doc = {
            "id" : raw_index.documentStore.docs[key].id,
            "title" : raw_index.documentStore.docs[key].title,
            "body" : raw_index.documentStore.docs[key].body,
            "link" : raw_index.documentStore.docs[key].link
          };
          index.addDoc(doc);
        }
        
        return {index : index, docs : doc_list};
      });
  })
  
  .declareMethod("get_attachment", function(doc, att, format){
    return this.state.docs.getAttachment(doc, att, format);
  })
  
  .declareMethod("add_doc", function(new_doc){ //tested OK
    return this.state.docs.put(new_doc, {links : []});
  })
  
  .declareMethod("add_feed_to_doc", function(doc_name, feed){ //tested OK
    var gadget = this;
    return gadget.state.docs.putAttachment(doc_name, feed.title, new Blob([feed.content], {type : "text/xml"}))
      .push(function(){
        return gadget.state.docs.get(doc_name);
      })
      .push(function(doc){
        var link_list = doc.links;
        link_list.push(feed.title);
        return gadget.state.docs.put(doc_name, {links : link_list});
      });
  })
  
  .declareMethod("add_index", function(name){ //tested OK
    var new_index = elasticlunr(function () {
        this.use(elasticlunr.fr);
        this.addField('title');
        this.addField('body');
        this.addField('link');
        this.setRef('id'); //id = index.x."/a/b" where index = index name, x = doc name, "/a/b" = item id in x  
    });
    return this.state.indexes.put(name, {index : JSON.stringify(new_index), docs : []});
  })
  
  .declareMethod("add_doc_to_index", function(index_name, doc_name){ //tested OK - to split & clean
    var index,
        parser_list = [],
        gadget = this;
    return gadget.get_index(index_name)
      .push(function(result_index){
        index = result_index;
      })
      .push(function(){
        return gadget.get_doc(doc_name);
      })
      .push(function(result_doc){
        var attachment_list = result_doc.links,
            i,
            parser,
            promise_list = [];
        for (i=0; i<attachment_list.length; i+=1){
          var parser_def = {
              type : "my_parser",
              document_id : doc_name,
              attachment_id : attachment_list[i],
              parser : "rss",
              sub_storage : {
                type : "indexeddb",
                database : "mynij.all_docs"
              }
            };
          parser = jIO.createJIO(parser_def);
          parser_list.push(parser);
          promise_list.push(parser.allDocs());
        }
        return RSVP.all(promise_list); //returns [{data : {rows : [id : _], total_rows : _}]
      })
      .push(function(all_item_ids){
        var i,
            j,
            promise_list = [];
        for (i=0; i<all_item_ids.length; i+=1){
          for (j=0; j<all_item_ids[i].data.total_rows; j+=1){
            promise_list.push(parser_list[i].get(all_item_ids[i].data.rows[j].id));
          }
        }
        return RSVP.all(promise_list); //returns [{link : _, title : _, description : _, pubDate : _, ...}] ! ignore [0], it's the feed info !
      })
      .push(function(all_items){
        var i,
            id,
            doc;
        for (i=1; i<all_items.length; i+=1){
          id = index_name+"."+doc_name+"./0/"+i;
          doc = {
            "id" : id,
            "title" : all_items[i].title,
            "body" : all_items[i].description,
            "link" : all_items[i].link
          };
          index.index.addDoc(doc);
        }
      })
      .push(function(){
        var doc_list = index.docs;
        doc_list.push(doc_name);
        return gadget.state.indexes.put(index_name, {index : JSON.stringify(index.index), docs : doc_list});
      });
  })
  
  .declareMethod("loaded", function(){
    return this.state.docs.allDocs()
      .push(function(result){
        return (result.data.rows.length !== 0);
      });
  });

}(window, RSVP, rJS, jIO));